// globaldefines.h ***********************************************************
//
// Simple transparent data converter for STM32L476 Nucleo-64
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Additionally, a number of user defined MIDI messages can be released by
// attached pushbuttons or potentiometers.
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.h", "main.h", "userinterface.c",
// and "usb-otg.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction 
// and operation manual is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define HARDWARE_VERSION 3  //describes project specific hardware version
#define REVNUM 6            //describes firmware revision
//  #########################################################################
// Version changes:
//REVNUM 2	is the first release for STM32L476
//REVNUM 3:	Vusb sensing enabled, therefore USART1 TX PA9 remapped to PB6
//			for this reason hardware modified to version2, doesn't work with HW1
//REVNUM 4:	 MIDI Baudrate added for USART1 in USB CDC and MIDI mode
//			new: factoryReset()
//REVNUM 5:	new 06Oct2021
//			USB interface improved
//			Flash storage improved
//			User interface ("CMD>") improved
//			internal handling of input change messages improved
//			development migrated to Segger Embedded Studio for ARM 5.44
//			unmodified Nucleo: USART2 is active in parallel with USART1
//release 24 Oct 21
//REVNUM 6  new21Nov2
//21Nov21	new HARDWARE_VERSION 3, more I/O externally available
//			USB stability improved
//			user interface improved, decimal number base supported
//02Dec21	USART2 handling improved

//  #########################################################################

#define STLINK_CUTOFF
//the onboard ST-Link module is cut off or not used

#ifndef STLINK_CUTOFF
// if NOT defined:
// unmodified Nucleo-L476

//#define STLINK_USB
// then STM32L476 internal USB is NOT active
// and USB communication is redirceted via USART2 (driven by ST-LINK module)
// else Veroboard USB is active and USART2 works in parallel with USART1
#endif
// *****************************

//the device will appear as "STMicroelectronics Virtual Com Port"
#define DEFAULT_VIDH	0x04
#define DEFAULT_VIDL	0x83
#define DEFAULT_VID		((DEFAULT_VIDH << 8) | DEFAULT_VIDL)
#define DEFAULT_PIDH	0x57
#define DEFAULT_PIDL	0x40
#define DEFAULT_PID		((DEFAULT_PIDH << 8) | DEFAULT_PIDL)

#define IO_NUM	17		//number of inputs for change messages
						//incl User Button

#define MIDICOM_USART1      // USART1 (PortA10, PortB6) gets activated
#ifndef MIDICOM_USART1
 #define MIDICOM_USART2     // USART2 (PortA  2,3) gets activated
//when USART2 is seleceted, serial communication takes place via ST-Link/USB
//for compatibility reasons USART2 operates with 115200 Baud even in MIDI mode
//this is well for test when no legacy COM port is available,
//but it cannot be made compatible with MIDI software running on the PC !
#endif

#define CDC_CLASS   0       //initatalizes device as virtual COM port
#define MIDI_CLASS  1       //initatalizes device as native MIDI interface

#define EOX     0xF7
#define RUNNINGSTATE  //uncomment if not wanted
//works only on MIDI msgs which are transmitted from MIDI OUT
//Running State is always supported with MIDI msgs received at MIDI IN

//bitmask shortcuts:
#define m0      1
#define m1      (1<<1)
#define m2      (1<<2)
#define m3      (1<<3)
#define m4      (1<<4)
#define m5      (1<<5)
#define m6      (1<<6)
#define m7      (1<<7)
#define m8      (1<<8)
#define m9      (1<<9)
#define m10     (1<<10)
#define m11     (1<<11)
#define m12     (1<<12)
#define m13     (1<<13)
#define m14     (1<<14)
#define m15     (1<<15)
#define m16     (1<<16)
#define m17     (1<<17)
#define m18     (1<<18)
#define m19     (1<<19)
#define m20     (1<<20)
#define m21     (1<<21)
#define m22     (1<<22)
#define m23     (1<<23)
#define m24     (1<<24)
#define m25     (1<<25)
#define m26     (1<<26)
#define m27     (1<<27)
#define m28     (1<<28)
#define m29     (1<<29)
#define m30     (1<<30)
#define m31     (1<<31)

#endif
