// main.h *******************************************************************
//
// Simple transparent data converter for STM32L476 Nucleo-64
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "userinterface.c",
// "usb-otg.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction 
// and operation manual is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "globaldefines.h"

// ************************************
//from userinterface.c
extern void userConfig(void);
//from usb-fs.c  ************
extern uint32_t guDeviceClass, guConfigured;
extern void stopUSB(void);
extern void initUSB(void);
extern uint16_t pullUsbRxBuf(void);
extern void sendUsbByte(uint8_t);
extern void enCodeMidi(uint8_t);
// ****************************************

#define USART_BUFLEN 4094
uint8_t gUSART1_RXBUF[USART_BUFLEN+2];
uint8_t *gUSART1_RXHEAD=0,*gUSART1_RXTAIL=0,*gUSART1_RXBUFTOP= &(gUSART1_RXBUF[USART_BUFLEN]);
#ifndef STLINK_CUTOFF
uint8_t gUSART2_RXBUF[USART_BUFLEN+2];
uint8_t *gUSART2_RXHEAD=0,*gUSART2_RXTAIL=0,*gUSART2_RXBUFTOP= &(gUSART2_RXBUF[USART_BUFLEN]);
#endif
//system control
uint32_t gSYSCOUNT, gGREEN_LEDCOUNT, gRED_LEDCOUNT;
uint32_t gDELAYCOUNT, gTIMEOUT, gGUARDTIME, gUSB_GUARDTIME;
//gHGUARDTIME and gHStartCmd work on USB=Host byte stream
uint32_t gStartSetup, gUsbStartSetup, gSetupMode;
uint32_t gExecOn, gExecBlock, gAdcOn, gFlashChanged, gNumBase;
uint8_t gGlobalMidiChannel;

uint32_t gTest;		//for debugging only
uint32_t gOver;    //exclusively used for SETUP_DBG

#define PAGE15 (0x08007800)

struct {            //half words for code compatibility with STM32F042,
uint16_t ComBaud;	//which allows only half word flash write
uint16_t MidiBaud;
uint16_t Channel;
uint16_t Hysteresis;
uint16_t VidH;
uint16_t VidL;
uint16_t PidH;
uint16_t PidL;
//the offset for a MIDI msg entry relative to "PidH" is = (user I/Onum *2)
uint16_t Msg1S;
uint16_t Msg1D;
uint16_t Msg2S;
uint16_t Msg2D;
uint16_t Msg3S;
uint16_t Msg3D;
uint16_t Msg4S;
uint16_t Msg4D;
uint16_t Msg5S;
uint16_t Msg5D;
uint16_t Msg6S;
uint16_t Msg6D;
uint16_t Msg7S;
uint16_t Msg7D;
uint16_t Msg8S;
uint16_t Msg8D;
uint16_t Msg9S;
uint16_t Msg9D;
uint16_t Msg10S;
uint16_t Msg10D;
uint16_t Msg11S;
uint16_t Msg11D;
uint16_t Msg12S;
uint16_t Msg12D;
uint16_t Msg13S;
uint16_t Msg13D;
uint16_t Msg14S;
uint16_t Msg14D;
uint16_t Msg15S;
uint16_t Msg15D;
uint16_t Msg16S;
uint16_t Msg16D;
uint16_t MsgUS;
uint16_t MsgUD;
uint16_t NumBase;	
uint16_t dummy;	//dummy necessary: flash is programmed in double words
} gFlashMirror,gRuntimeSetup;//44 halfwords= 88bytes= 22 words
//gFlashMirror contains modified setup to be written to Flash
//gRuntimeSetup contains active working setup, possibly changed during session

uint16_t gInTest[IO_NUM +1];   //index 0 is never used!
uint16_t gInState[IO_NUM +1];
uint8_t gHysteresis[IO_NUM +1];

const uint8_t gTypeString [] = "USB-MidiCom v" ;
const uint8_t gCopyString [] = " (c)2016-21 W.Schemmert";

//static void SetSysClock(void);
void configure_GPIO(void);
void configurePort(uint32_t);
static void SetSysClock(void);
void LED_Off(void);
void msDelay(uint16_t) ;
void servSystem(void);
void MsgExecutor(void);
void debounceAnalog(uint8_t, uint16_t);
void debounceDigital(uint8_t,uint16_t);
uint16_t readADC(uint8_t);
//void calibrateADC(void);
void initADC(void);
void sendAnaMsg(uint8_t);
void sendDigMsg(uint8_t);
void sendAsciiMsg(uint8_t, uint8_t, uint16_t);
void popFlashPage15(void);
void pushFlashPage15(void);
//USART
void initUsart1(void);
void initUsart2(void);
void sendB(uint8_t);
void sendU2B(uint8_t);
void sendAllB(uint8_t);
void sendEncodeB(uint8_t);
void sendX8(uint8_t);
void sendAllX8(uint8_t);
void sendNibble(uint8_t);
void sendAllNibble(uint8_t);
void sendAllX12(uint16_t);
void sendAllS12(uint16_t);
void sendAllX16(uint16_t);
void sendX32(uint32_t);
void sendAllString(const uint8_t *);
void sendCRLF(void);
uint32_t pullUsart1RxBuf(void);
uint32_t pullUsart2RxBuf(void);
uint32_t pullAllRxBuf(void);

#ifdef SETUP_DBG
uint8_t gDBGBUF[4096];  //4096 bytes
uint8_t *gDBGHEAD=gDBGBUF,*gDBGBUFTOP=&(gDBGBUF[4096]);

void initSDBG(void);
void sdbgB(uint8_t);
void sdbgX8(uint8_t);
void sdbgNibble(uint8_t);
void sdbgX16(uint16_t);
void sdbgX32(uint32_t);
void sendDebug(void);
#endif

//Interrupt handlers
void SysTick_Handler(void);
void USART1_IRQHandler(void);
void USART2_IRQHandler(void);
